@@ -12,16 +12,17 @@
 //!
 //! ### Usage in Applications
 //!
-//! Applications configure their tracer either by [installing a trace pipeline],
+//! Applications configure their tracer either by installing a trace pipeline,
 //! or calling [`set_tracer_provider`].
 //!
 //! ```
 //! # #[cfg(feature="trace")]
 //! # {
-//! use opentelemetry::trace::{Tracer, NoopTracerProvider};
+//! use opentelemetry::trace::{Tracer, noop::NoopTracerProvider};
 //! use opentelemetry::global;
 //!
 //! fn init_tracer() {
+//!     // Swap this no-op provider for your tracing service of choice (jaeger, zipkin, etc)
 //!     let provider = NoopTracerProvider::new();
 //!
 //!     // Configure the global `TracerProvider` singleton when your app starts
@@ -39,7 +40,7 @@
 //! }
 //!
 //! // in main or other app start
-//! let _ = init_tracer();
+//! init_tracer();
 //! do_something_tracked();
 //! # }
 //! ```
@@ -49,13 +50,16 @@
 //! ```
 //! # #[cfg(feature="trace")]
 //! # {
-//! use opentelemetry::trace::Tracer;
+//! use opentelemetry::trace::{Tracer, TracerProvider};
 //! use opentelemetry::global;
 //!
 //! pub fn my_traced_library_function() {
 //!     // End users of your library will configure their global tracer provider
 //!     // so you can use the global tracer without any setup
-//!     let tracer = global::tracer_with_version("my-library-name", env!("CARGO_PKG_VERSION"));
+//!     let tracer = global::tracer_provider().tracer_builder("my-library-name").
+//!         with_version(env!("CARGO_PKG_VERSION")).
+//!         with_schema_url("https://opentelemetry.io/schemas/1.17.0").
+//!         build();
 //!
 //!     tracer.in_span("doing_library_work", |cx| {
 //!         // Traced library logic here...
@@ -64,7 +68,6 @@
 //! # }
 //! ```
 //!
-//! [installing a trace pipeline]: crate::sdk::export::trace::stdout::PipelineBuilder::install_simple
 //! [`TracerProvider`]: crate::trace::TracerProvider
 //! [`Span`]: crate::trace::Span
 //!
@@ -78,60 +81,47 @@
 //! written against this generic API and not constrain users to a specific
 //! implementation choice.
 //!
-//! ### Usage in Applications
+//! ### Usage in Applications and libraries
 //!
-//! Applications configure their meter either by [installing a metrics pipeline],
-//! or calling [`set_meter_provider`].
+//! Applications and libraries can obtain meter from the global meter provider,
+//! and use the meter to create instruments to emit measurements.
 //!
 //! ```
 //! # #[cfg(feature="metrics")]
 //! # {
-//! use opentelemetry::metrics::{Meter, noop::NoopMeterProvider};
+//! use opentelemetry::metrics::{Meter};
 //! use opentelemetry::{global, KeyValue};
 //!
-//! fn init_meter() {
-//!     let provider = NoopMeterProvider::new();
-//!
-//!     // Configure the global `MeterProvider` singleton when your app starts
-//!     // (there is a no-op default if this is not set by your application)
-//!     global::set_meter_provider(provider)
-//! }
-//!
-//! fn do_something_instrumented() {
-//!     // Then you can get a named tracer instance anywhere in your codebase.
+//!    fn do_something_instrumented() {
 //!     let meter = global::meter("my-component");
+//!     // It is recommended to reuse the same counter instance for the
+//!     // lifetime of the application
 //!     let counter = meter.u64_counter("my_counter").init();
 //!
-//!     // record metrics
+//!     // record measurements
 //!     counter.add(1, &[KeyValue::new("mykey", "myvalue")]);
+//!     }
 //! }
-//!
-//! // in main or other app start
-//! init_meter();
-//! do_something_instrumented();
-//! # }
 //! ```
 //!
-//! ### Usage in Libraries
-//!
+//! ### Usage in Applications
+//! Application owners have the responsibility to set the global meter provider.
+//! The global meter provider can be set using the [`set_meter_provider`] function.
+//! As set_meter_provider takes ownership of the provider, it is recommended to
+//! provide a clone of the provider, if the application needs to use the provider
+//! later to perform operations like shutdown.
 //! ```
 //! # #[cfg(feature="metrics")]
 //! # {
 //! use opentelemetry::{global, KeyValue};
 //!
-//! pub fn my_traced_library_function() {
-//!     // End users of your library will configure their global meter provider
-//!     // so you can use the global meter without any setup
-//!     let tracer = global::meter("my-library-name");
-//!     let counter = tracer.u64_counter("my_counter").init();
-//!
-//!     // record metrics
-//!     counter.add(1, &[KeyValue::new("mykey", "myvalue")]);
+//! fn main() {
+//!    // Set the global meter provider
+//!    // global::set_meter_provider(my_meter_provider().clone());
 //! }
 //! # }
 //! ```
 //!
-//! [installing a metrics pipeline]: crate::sdk::export::metrics::stdout::StdoutExporterBuilder::init
 //! [`MeterProvider`]: crate::metrics::MeterProvider
 //! [`set_meter_provider`]: crate::global::set_meter_provider
 
@@ -146,16 +136,10 @@ mod trace;
 pub use error_handler::{handle_error, set_error_handler, Error};
 #[cfg(feature = "metrics")]
 #[cfg_attr(docsrs, doc(cfg(feature = "metrics")))]
-pub use metrics::{
-    meter, meter_provider, meter_with_version, set_meter_provider, GlobalMeterProvider,
-};
+pub use metrics::*;
 #[cfg(feature = "trace")]
 #[cfg_attr(docsrs, doc(cfg(feature = "trace")))]
-pub use propagation::{get_text_map_propagator, set_text_map_propagator};
+pub use propagation::*;
 #[cfg(feature = "trace")]
 #[cfg_attr(docsrs, doc(cfg(feature = "trace")))]
-pub use trace::{
-    force_flush_tracer_provider, set_tracer_provider, shutdown_tracer_provider, tracer,
-    tracer_provider, tracer_with_version, BoxedSpan, BoxedTracer, GenericTracer,
-    GenericTracerProvider, GlobalTracerProvider,
-};
+pub use trace::*;
